#!/usr/bin/env python
# -*- encoding: utf-8 -*-

# Feb. 2016, wwortel
# version: 0.06

import sys, os, re
from sys import platform as _platform

####################################
# EDIT THE NAME OF THE RESULTING FILE
####################################
res_name = 'dvb_s_mw_s1_new'

####################################
# EDIT FROM WHICH FOLDERS TO COMPOSE
# THE NEW mw_s1, comma separated
####################################
sfolders = ([ 
  'services_ordered_27_5_W_TV' ,
  'services_ordered_27_5_W_Radio' ,
  'services_ordered_30_0_W_TV' ,
  'services_ordered_30_0_W_Radio' 
])
####################################

dhere = os.path.realpath('.')
resultfile = os.path.join(dhere, res_name)
tmpfile = os.path.join(dhere, 'tmpbinary')

if _platform == "linux" or _platform == "linux2": #  or _platform == "darwin":
  # linux
  slash = '/'
  pycrcpath = dhere + slash + 'pycrc-0.8.3-linux'
elif _platform == "win32":
  # Windows
  slash = "\\" # a single \ would make the " literal
  pycrcpath = dhere + slash + 'pycrc-0.8.3-win32'

pycrcpath = os.path.join(pycrcpath, 'pycrc.py')

# pycrc package from pycrc.org under MIT License

def atoi(text):
  return int(text) if text.isdigit() else text

def natural_keys(text):
  '''
  alist.sort(key=natural_keys) sorts in human order
  http://nedbatchelder.com/blog/200712/human_sorting.html
  (See Toothy's implementation in the comments)
  '''
  return [ atoi(c) for c in re.split('(\d+)', text) ]

satservices = bytearray()  # New empty byte array
servicecount = 0

def error(fmt, *msg):
  sys.stderr.write(" [ERROR] " + (fmt % msg) + '\n')
  sys.exit(1)

def pmsg(fmt, *msg):
  print(fmt % msg)

# function to receive a int/long 'decimal' and return bytearray with 'count'
# bytes, big endian and counting bytes from LSB side 
def int2ba(decimal, count):
  result = bytearray(count)
  j = 8 * ( count - 1)
  for i in range(0 ,count,1):
    result[i] = int( 0xff & decimal >> j)
    j -=8
  return result

# Python 3.x requires encoding info for the bytearray method
if sys.hexversion < 0x03000000:
    def str2ba(string):
      return bytearray(string)
else:
    def str2ba(string):
      return bytearray(string, "utf-8")

for odir in sfolders:
  dthere = dhere + slash
  dthere = os.path.join(dhere, odir)
  dthere = os.path.realpath(dthere)
  for fname in sorted(os.listdir(dthere), key=natural_keys):
    fullpath = os.path.join(dthere, fname)
# get the preference setting from the end of the file's name
# regex search for decimals at the end after a '-'
    try:
      pref = re.search(".*~P(\d+$)", fname).group(1)
    except:
      error('no preference found at end of "%s"; exit!', fname)
      exit
    with open(str(fullpath), 'rb') as binfile:
      sdata = bytearray(binfile.read())
# adapt the preference setting in the data record; always do this, even when zero
    prefba = int2ba( int(pref), 2) # make a 2 byte array from the pref text string
    rcdlen = len(sdata)
    sdata[ rcdlen - 10 : rcdlen - 8 ] = prefba
# regex search for ~E1~ in the file name; if so replace service name
    try:
      pref = re.search("^.*~\d+~.*~(E.)~.*$", fname).group(1)
    except:
      error('no edit flag found in filename: "%s"; exit!', fname)
      exit
    if pref == "E1":
# isolate the new service name
      try:
        newn = str(re.search("^.*~\d+~(.*)~E.*$", fname).group(1))
      except:
        error('could not isolate the service name in filename: "%s"; exit!', fname )
        exit      
# set the bit that locks the data against automatic updates in the background
      sdata[ rcdlen - 8 ] = 0x01
# swap spaces back in i.s.o. single underscores; two consecutive underscores make one underscore
      newn = re.sub(r"__", "<>><", newn) # unlikely placeholder for the underscore
      newn = re.sub(r"_", " ", newn) # swap single underscores
      newn = re.sub(r"<>><", "_", newn) # make the placeholder an undersocre.
      newl = len(newn)
# get the original name length
      rcdnamel = int(sdata[1])
# cut the record in three
      filler=sdata[2:2+3]
      payload=sdata[rcdnamel+5:]
      newrec=bytearray(( int(0x01) , int(newl) ))
      newrec.extend(filler)
      newrec.extend(str2ba(newn))
      newrec.extend(payload)
      sdata=newrec # service name changes in new listing but remains original in the file
# concatenate this service to the earlier ones 
    satservices.extend(sdata)
    servicecount += 1
pmsg('Total number of services: %d.', servicecount)

# A: file format; a fixed type $0c
ffbytes = bytearray((int(0x00), int(0x00),int(0x00), int(0x0C)))

# B: number of services in file
snbytes = int2ba(servicecount, 4)

# C: length minus 4 bytes of length itself
fl = len(satservices) + 12 # 4 crc + 4 type + 4 service count + all service records
flbytes = int2ba(fl, 4)

# D: 
# http://stackoverflow.com/questions/4594899/how-do-you-convert-a-decimal-number-into-a-list-of-bytes-in-python
# make intermediate data file for crc calculation
bindata = bytearray()
bindata.extend(ffbytes)
bindata.extend(snbytes)
bindata.extend(satservices)

with open(str(tmpfile), 'wb') as servicesf:
  servicesf.write(bindata)

crcbytes = bytearray(4) # New empty 4 byte array
# here lots of '"' are needed to not let Windows choke on spaces in file paths.
crcresult=os.popen('python ' + '"' + pycrcpath + '"' + ' --model=crc-32-mpeg --check-file=' + '"' + tmpfile + '"' ).read()

# strip leading 0x from crc result string and pad with 0 to the left to force 8 characters as we need 4 crc bytes,
# pycrc's output is variable string length, omitting insignificant zeros
crcresult = crcresult[2:].zfill(8)
# now transform 4 character pairs in the string to 4 bytes
for i in range(0,8,2):
  value = crcresult[i:i+2]
  crcbytes[ int (i / 2) ] = int('0x' + value,16)

# Moment suprème, reconstruct mw_s1
mw_s1 = bytearray()
mw_s1.extend(flbytes)
mw_s1.extend(crcbytes)
mw_s1.extend(bindata)

# and write it to file
with open(str(resultfile), 'wb') as servicesf:
  servicesf.write(mw_s1)

# remove the temporary file
os.remove(str(tmpfile))
pmsg('Result file written: %s.', resultfile)
pmsg('Rename it to mw_s1, for transfer via SD card, or to dvb_s_mw_s1 for sftp transfer.')
